@@ -61,6 +61,7 @@ sub ACTION_code {
       my $unpack;
       $unpack = (-d "$build_src/iup") ? $self->prompt("\nDir '$build_src/iup' exists, wanna replace with clean sources?", "n") : 'y';
       if (lc($unpack) eq 'y') {
+        File::Path::rmtree("$build_src/iup") if -d "$build_src/iup";
         $self->prepare_sources($self->notes('iup_url'), $self->notes('iup_sha1'), $download, $build_src);
         if ($self->notes('iup_patches')) {
           $self->apply_patch("$build_src/iup", $_)  foreach (@{$self->notes('iup_patches')});
@@ -69,6 +70,7 @@ sub ACTION_code {
 
       $unpack = (-d "$build_src/im") ? $self->prompt("\nDir '$build_src/im'  exists, wanna replace with clean sources?", "n") : 'y';
       if (lc($unpack) eq 'y') {
+        File::Path::rmtree("$build_src/im") if -d "$build_src/im";
         $self->prepare_sources($self->notes('im_url'), $self->notes('im_sha1'), $download, $build_src);
         if ($self->notes('im_patches')) {
           $self->apply_patch("$build_src/im", $_)  foreach (@{$self->notes('im_patches')});
@@ -77,17 +79,28 @@ sub ACTION_code {
 
       $unpack = (-d "$build_src/cd") ? $self->prompt("\nDir '$build_src/cd'  exists, wanna replace with clean sources?", "n") : 'y';
       if (lc($unpack) eq 'y') {
+        File::Path::rmtree("$build_src/cd") if -d "$build_src/cd";
         $self->prepare_sources($self->notes('cd_url'), $self->notes('cd_sha1'), $download, $build_src);
         if ($self->notes('cd_patches')) {
           $self->apply_patch("$build_src/cd", $_)  foreach (@{$self->notes('cd_patches')});
         }
       }
 
-      my $m = $self->prompt("\nDo you want to see all messages during 'make' (y/n)?", 'n');
+      my $m = $self->notes('build_debug_info') ? $self->prompt("\nDo you want to see all messages during 'make' (y/n)?", 'n') : 'n';
       $self->notes('build_msgs', lc($m) eq 'y' ? 1 : 0);
 
       # go for build
-      $self->build_binaries($build_out, $build_src);
+      my $success = $self->build_binaries($build_out, $build_src);
+      my $done    = $self->config_data('info_done');
+      my $iuplibs = $self->config_data('iup_libs');
+      if ($self->notes('build_debug_info')) {
+        print STDERR "Build result: $done->{$_} - $_\n" foreach (sort keys %$done);
+        print STDERR "Output libs : $iuplibs->{$_} - $_\n" foreach (sort keys %$iuplibs);
+      }
+      die "###BUILD FAILED### essential libs (iup/im/cd) not built!" unless $done->{"iup:iup"} && $done->{"iup:iupim"} && $done->{"iup:iupcd"};
+      die "###BUILD FAILED###" unless $success;
+      #DEBUG: die intentionally at this point if you want to see build details from cpan testers  
+      print STDERR "RESULT: OK!\n";
 
       # store info about build to ConfigData
       $self->config_data('share_subdir', $self->{properties}->{dist_version});
@@ -109,9 +122,9 @@ sub ACTION_code {
 sub prepare_sources {
   my ($self, $url, $sha1, $download, $build_src) = @_;
   $self->fetch_file($url, $sha1, $download);
-   my $archive = catfile($download, File::Fetch->new(uri => $url)->file);
-   my $ae = Archive::Extract->new( archive => $archive );
-   die "###ERROR### Cannot extract tarball ", $ae->error unless $ae->extract(to => $build_src);
+  my $archive = catfile($download, File::Fetch->new(uri => $url)->file);
+  my $ae = Archive::Extract->new( archive => $archive );
+  die "###ERROR### Cannot extract tarball ", $ae->error unless $ae->extract(to => $build_src);
 }
 
 sub fetch_file {
@@ -144,8 +157,9 @@ sub check_sha1sum {
   binmode($fh);
   $sha1->addfile($fh);
   close($fh);
-  my $rv = ($sha1->hexdigest eq $sha1sum) ? 1 : 0;
-  warn "###WARN## sha1 mismatch: got      '", $sha1->hexdigest , "'\n",
+  my $file_sha1sum = $sha1->hexdigest;
+  my $rv = ($file_sha1sum eq $sha1sum) ? 1 : 0;
+  warn "###WARN## sha1 mismatch: got      '", $file_sha1sum , "'\n",
        "###WARN## sha1 mismatch: expected '", $sha1sum, "'\n",
        "###WARN## sha1 mismatch: filesize ", (-s $file) unless $rv;
   return $rv;
@@ -163,6 +177,12 @@ sub quote_literal {
 
 sub check_installed_lib {
   my ($self) = @_;
+  
+  #xxxTODO
+  #we not only need to detect the presence we also need to exactly know what libs are there - necessary for havelib() function
+  print STDERR "\nNOTICE:\nDetection of preinstalled iup+cd+im is disabled since v0.115!\nPlease contact the module author if you are missing this feature.\n\n";
+  return 0;
+  
   my $idir = $ENV{IUP_DIR} || '';
   my @candidates;
   push(@candidates, { L => "$idir/lib", I => "$idir/include" }) if -d $idir;
@@ -221,15 +241,11 @@ int demofunc(void) { return 0; }
 
 MARKER
   close($fs);
-  local *OLDERR;
-  open OLDERR, ">&", STDERR;
-  open STDERR, ">", File::Spec->devnull();
   $src = $self->quote_literal($src);
   $obj = $self->quote_literal($obj);
   #Note: $Config{cc} might contain e.g. 'ccache cc' (FreeBSD 8.0)
-  my $rv = system("$Config{cc} -c -o $obj $src $cflags");
-  open(STDERR, ">&", OLDERR);
-  return ($rv == 0) ? 1 : 0;
+  my $rv = run3("$Config{cc} -c -o $obj $src $cflags", \undef, \undef, \undef, { return_if_system_error => 1 } );
+  return ($rv == 1 && $? == 0) ? 1 : 0;
 }
 
 # check presence of lib(s) specified as params
@@ -240,6 +256,7 @@ sub check_lib {
   $cflags =~ s/[\r\n]//g;
   $lflags =~ s/[\r\n]//g;
   my @libs = ref($l) ? @$l : ( $l );
+  my $liblist = scalar(@libs) ? '-l' . join(' -l', @libs) : '';
 
   my ($fs, $src) = File::Temp->tempfile('tmpfileXXXXXXaa', SUFFIX => '.c', UNLINK => 1);
   my ($fo, $obj) = File::Temp->tempfile('tmpfileXXXXXXaa', SUFFIX => '.o', UNLINK => 1);
@@ -249,21 +266,25 @@ int main() { return 0; }
 
 MARKER
   close($fs);
-  local *OLDERR;
-  open OLDERR, ">&", STDERR;
-  open STDERR, ">", File::Spec->devnull();
   $src = $self->quote_literal($src);
   $obj = $self->quote_literal($obj);
   $exe = $self->quote_literal($exe);
+  my $output;
   #Note: $Config{cc} might contain e.g. 'ccache cc' (FreeBSD 8.0)
-  my $rv1 = system("$Config{cc} -c -o $obj $src $cflags");
-  my $liblist = scalar(@libs) ? '-l' . join(' -l', @libs) : '';
-  my $rv2 = ($rv1 == 0) ? system("$Config{ld} $obj -o $exe $lflags $liblist") : -1;
-  open(STDERR, ">&", OLDERR);
-  return ($rv2 == 0) ? 1 : 0;
+  my $rv1 = run3("$Config{cc} -c -o $obj $src $cflags", \undef, \$output, \$output, { return_if_system_error => 1 } );
+  unless ($rv1 == 1 && $? == 0) {
+    #print STDERR "OUTPUT(compile):\n$output\n" if $output;
+    return 0
+  }
+  my $rv2 = run3("$Config{ld} $obj -o $exe $lflags $liblist", \undef, \$output, \$output, { return_if_system_error => 1 } );
+  unless ($rv2 == 1 && $? == 0) {
+    #print STDERR "OUTPUT(link):\n$output\n" if $output;
+    return 0
+  }
+  return 1;
 }
 
-# pude perl implementation of patch functionality
+# pure perl implementation of patch functionality
 sub apply_patch {
   my ($self, $dir_to_be_patched, $patch_file) = @_;
   my ($src, $diff);
@@ -283,7 +304,7 @@ sub apply_patch {
     $k =~ s|\\|/|g;
     $k =~ s|^[^/]*/(.*)$|$1|;
     $k = catfile($dir_to_be_patched, $k);
-    print STDERR "Gonna patch file '$k'\n" if $self->notes('build_debug_info');
+    print STDERR "- gonna patch '$k'\n" if $self->notes('build_debug_info');
 
     open(SRC, $k) or die "###ERROR### Cannot open file: '$k'\n";
     $src  = <SRC>;
@@ -306,7 +327,7 @@ sub run_output_tail {
   my ($self, $limit, @cmd) = @_;
   my $output;
   print STDERR "CMD: " . join(' ',@cmd) . "\n";
-  print STDERR "Running (stdout+stderr redirected)...\n";
+  print STDERR "- running (stdout+stderr redirected)...\n";
   my $rv = run3(\@cmd, \undef, \$output, \$output, { return_if_system_error => 1 } );
   my $success = ($rv == 1 && $? == 0) ? 1 : 0;
   $output = substr $output, -$limit if defined $limit; # we want just last N chars
@@ -323,11 +344,11 @@ sub run_output_on_error {
   my ($self, $limit, @cmd) = @_;
   my $output;
   print STDERR "CMD: " . join(' ',@cmd) . "\n";
-  print STDERR "Running (stdout+stderr redirected)...\n";
+  print STDERR "- running (stdout+stderr redirected)...\n";
   my $rv = run3(\@cmd, \undef, \$output, \$output, { return_if_system_error => 1 } );
   my $success = ($rv == 1 && $? == 0) ? 1 : 0;
   if ($success) {
-    print STDERR "Finished successfully (output suppressed)\n";
+    print STDERR "- finished successfully (output suppressed)\n";
   }
   else {
     $output = substr $output, -$limit if defined $limit; # we want just last N chars
@@ -346,7 +367,7 @@ sub run_output_std {
   print STDERR "CMD: " . join(' ',@cmd) . "\n";
   my $rv = run3(\@cmd, undef, undef, undef, { return_if_system_error => 1 } );
   my $success = ($rv == 1 && $? == 0) ? 1 : 0;
-  print STDERR "Finished successfully\n" if ($success);
+  print STDERR "- finished successfully\n" if ($success);
   return $success;
 }
 
@@ -366,6 +387,19 @@ sub run_bothout2str {
   return $output;
 }
 
+sub run_custom {
+  my ($self, @cmd) = @_;
+  my $rv;
+  if ($self->notes('build_msgs')) {
+    $rv = $self->run_output_std(@cmd);
+  }
+  else {
+    $rv = $self->run_output_on_error($self->notes('build_msgs_limit'), @cmd);
+  }
+  warn "###WARN### error during run_custom()" unless $rv;
+  return $rv;
+}
+
 sub find_file {
   my ($self, $dir, $re) = @_;
   my @files;
@@ -379,7 +413,7 @@ sub find_file {
 
 sub sort_libs {
   my ($self, @unsorted) = @_;
-  my @wanted_order = qw/iupwin iupmot iupgtk iup iupcontrols iup_pplot iupcd iupgl iupim iupimglib cdwin cdx11 cdgdk cdgl cdpdf freetype6 freetype freetype-6 ftgl pdflib im im_fftw im_jp2 im_process/;
+  my @wanted_order = qw/iupcontrols iup_pplot iupcd iupgl iupim iupimglib iupole iupweb iuptuio iupwin iupmot iupgtk iup cdgl cdpdf cdwin cdx11 cdgdk cd freetype6 freetype freetype-6 ftgl pdflib im_fftw im_jp2 im_process im/;
   my @sorted;
   my %u;
 
@@ -399,5 +433,4 @@ sub sort_libs {
   return @sorted;
 }
 
-
 1;
