@@ -74,15 +74,35 @@ sub build_binaries {
     print STDERR "extra_cflags=$extra_cflags\n";
     print STDERR "extra_lflags=$extra_lflags\n";
   }
-  my %has;
 
-  $has{gtk}     = `pkg-config --modversion gtk+-2.0 2>/dev/null` ? 1 : 0;        #iupgtk
-  $has{gtkx11}  = `pkg-config --modversion gtk+-x11-2.0 2>/dev/null` ? 1 : 0;
-  $has{gdk}     = `pkg-config --modversion gdk-2.0 2>/dev/null` ? 1 : 0;        #cdgdk
-  $has{gdkx11}  = `pkg-config --modversion gdk-x11-2.0 2>/dev/null` ? 1 : 0;
-  $has{cairo}   = `pkg-config --modversion cairo 2>/dev/null` ? 1 : 0;                 #cdcairo
-  $has{pango}   = `pkg-config --modversion pango 2>/dev/null` ? 1 : 0;                #cdcairo
-  #$has{pangox}  = `pkg-config --modversion pangox 2>/dev/null` ? 1 : 0;
+  my %list = (
+    gtk        => 'gtk+-2.0',
+    gtkx11     => 'gtk+-x11-2.0',
+    gdk        => 'gdk-2.0',
+    gdkx11     => 'gdk-x11-2.0',
+    gtkprint   => 'gtk+-unix-print-2.0',
+    webkit     => 'webkit-1.0',
+    cairo      => 'cairo',
+    pango      => 'pango',
+    pangox     => 'pangox',
+    pangocairo => 'pangocairo',
+    freetype2  => 'freetype2',
+    x11        => 'x11',
+    xext       => 'xext',
+    xp         => 'xp',
+    xm         => 'xm',
+    xmu        => 'xmu',
+  );
+
+  my %has;
+  my %has_details;
+  for (sort keys %list) {
+    my $v = $self->run_stdout2str(qw[pkg-config --modversion], $list{$_}) || '';
+    my $p = $self->run_stdout2str(qw[pkg-config --variable=prefix], $list{$_}) || '';
+    $has{$_} = $v ? 1 : 0;
+    $has_details{$_} = { version=>$v, prefix=>$p };    
+    printf STDERR ("mod:% 20s version:% 9s prefix:%s\n", $list{$_}, $v, $p) if $self->notes('build_debug_info');
+  }
 
   $has{l_gtk}   = $has{gtk}    && $self->check_lib( [] , `pkg-config --cflags gtk+-2.0 2>/dev/null`,     `pkg-config --libs gtk+-2.0 2>/dev/null`);
   $has{l_gtkx11}= $has{gtkx11} && $self->check_lib( [] , `pkg-config --cflags gtk+-x11-2.0 2>/dev/null`, `pkg-config --libs gtk+-x11-2.0 2>/dev/null`);
@@ -101,25 +121,96 @@ sub build_binaries {
   $has{l_GL}    = $self->check_lib( 'GL',   $extra_cflags, $extra_lflags );
   $has{l_GLU}   = $self->check_lib( 'GLU',  $extra_cflags, $extra_lflags . ' -lGL -lm' );
   $has{l_glut}  = $self->check_lib( 'glut', $extra_cflags, $extra_lflags );
-  $has{l_gdi32} = $self->check_lib( 'gdi32' );  # cygwin only
-  $has{l_glu32} = $self->check_lib( 'glu32' );  # cygwin only
+  $has{'l_stdc++'} = $self->check_lib( 'stdc++', $extra_cflags, $extra_lflags );
 
   $has{Xm}      = $self->check_header('Xm/Xm.h',   $extra_cflags);
   $has{Xlib}    = $self->check_header('X11/Xlib.h',$extra_cflags); #iupgl cdx11
   $has{glx}     = $self->check_header('GL/glx.h',  $extra_cflags); #iupgl
   $has{glu}     = $self->check_header('GL/glu.h',  $extra_cflags);
   $has{gl}      = $self->check_header('GL/gl.h',   $extra_cflags);
-  $has{windows} = $self->check_header('windows.h');                #iupwin
 
-  if ($self->notes('build_debug_info')) {
-    #$has{fftw3}   = $self->check_header('fftw3.h',   $extra_cflags); #im_fftw3 = http://www.fftw.org/
-    #$has{wmsdk}   = $self->check_header('wmsdk.h');                  #im_wmv
-    #$has{ecw}     = $self->check_header('NCSECWClient.h');           #im_format_ecw = ECW (Enhanced Compression Wavelet) format
-    #$has{XxXxX}   = $self->check_header('XxXxX/XxXxX.h');            #non existing header
-    print STDERR "Has: $has{$_} - $_\n" foreach (sort keys %has);
+  #kind of a special hack
+  $has{freetype} = $self->check_header('ft2build.h', `pkg-config --cflags gtk+-2.0 gdk-2.0 2>/dev/null`);
+
+  my @x11_libs; # just base X11 libs
+  push(@x11_libs, 'X11')  if $has{l_X11};
+  push(@x11_libs, 'Xext') if $has{l_Xext};
+
+  my @opengl_libs;
+  push(@opengl_libs, 'GL')  if $has{l_GL};
+  push(@opengl_libs, 'GLU') if $has{l_GLU};
+
+  my @imtargets;
+  my @cdtargets;
+  my @iuptargets;
+  
+  #possible targets:  im im_process im_jp2 im_fftw im_capture im_avi im_wmv
+  #possible targets:  cd_freetype cd_ftgl cd cd_pdflib cdpdf cdgl cdcontextplus cdcairo
+  #possible targets:  iup iupcd iupcontrols iup_pplot iupgl iupim iupimglib iupweb iuptuio
+  
+  if ($self->notes('is_devel_cvs_version')) {
+    ### DEVEL BUILD ###
+    @imtargets  = qw[im im_process im_jp2 im_fftw im_capture];
+    @cdtargets  = qw[cd_freetype cd_ftgl cd cd_pdflib cdpdf cdgl]; #xxx add cdcontextplus
+    @iuptargets = qw[iup iupcd iupcontrols iup_pplot iupgl iupim iupimglib iupweb iuptuio];
+  }
+  else {
+    @imtargets  = qw[im];
+    @cdtargets  = qw[cd_freetype cd];
+    @iuptargets = qw[iup iupcd iupcontrols iup_pplot iupgl iupim iupimglib];
+    #if ($^O eq 'openbsd') {
+    #  warn "###WARN### skipping im_process on OpenBSD";
+    #  @imtargets = grep { $_ !~ /^(im_process)$/ } @imtargets;
+    #}
+    #if ($^O eq 'solaris') {
+    #  warn "###WARN### skipping iuptuio on Solaris";
+    #  @iuptargets = grep { $_ !~ /^(iuptuio)$/ } @iuptargets;
+    #}
+  }
+
+  @cdtargets  = grep { $_ !~ /^(cd_ftgl|cdgl)$/ } @cdtargets unless $has{l_GLU};
+  @iuptargets = grep { $_ !~ /^(iupgl)$/ } @iuptargets unless $has{glx};
+  @iuptargets = grep { $_ !~ /^(iupweb)$/ } @iuptargets unless $has{webkit};
+
+  #store debug info into ConfigData
+  $self->config_data('info_has', \%has);
+  $self->config_data('info_lib_details', \%has_details);
+  $self->config_data('info_imtargets', \@imtargets);
+  $self->config_data('info_cdtargets', \@cdtargets);
+  $self->config_data('info_iuptargets', \@iuptargets);
+
+  my @makeopts  = qw[NO_DYNAMIC=Yes USE_NODEPEND=Yes];
+  #my @makeopts  = qw[NO_STATIC=Yes USE_NODEPEND=Yes];
+
+  #choose GUI subsystem, priorities if multiple subsystems detected: 1. GTK, 2. X11/Motif
+  my @libs;
+  my @build_opts;
+  my $build_target;
+
+  push(@build_opts, 'GTK2') if ($has{gtk} && $has{cairo} && $has{Xlib});
+  push(@build_opts, 'X11/Motif') if ($has{Xlib} && $has{Xm});
+
+  if (scalar(@build_opts) == 1) {
+    $build_target = $build_opts[0];
+  }
+  elsif (scalar(@build_opts) > 1) {
+    my $n = 1;
+    my $msg = "\nYou have the following build options available:\n" .
+              join("\n", map ($n++ . ") $_", @build_opts)) .
+	      "\nWhat do you wanna build?";
+    my $i = $self->prompt($msg, 1);
+    $build_target = $build_opts[$i-1];
+    die "###ERROR### Wrong selection!" unless $build_target;
+  }
+  else {
+    warn "###WARNING### No supported GUI subsystem detected!\n";
+  }
+
+  if ($self->notes('build_debug_info') || !$build_target) {
+    print STDERR "has: $has{$_} - $_\n" foreach (sort keys %has);
 
     print STDERR "Brute force lookup:\n";
-    my $re = qr/\/(Xlib.h|Xm.h|gtk.h|glu.h|glut.h|gl.h|gtkprintunixdialog.h|libX11\.[^\d]*|libGL\.[^\d]*|libXm\.[^\d]*)$/;
+    my $re = qr/\/(Xlib.h|Xm.h|gtk.h|cairo.h|glu.h|glut.h|gl.h|freetype.h|gtkprintunixdialog.h|jasper.h|jas_image.h|lib(X11|GL|Xm|freetype)\.[^\d]*)$/;
     print STDERR "[/usr    ] $_\n" foreach ($self->find_file('/usr', $re));
     print STDERR "[/lib    ] $_\n" foreach ($self->find_file('/usr', $re));
     print STDERR "[/opt    ] $_\n" foreach ($self->find_file('/opt', $re));
@@ -137,77 +228,33 @@ sub build_binaries {
       print STDERR "[includedir $pkg] " . $self->run_stdout2str(qw[pkg-config --variable=includedir], $pkg) . "\n";
     }
   }
-
-  my @x11_libs; # just base X11 libs
-  push(@x11_libs, 'X11')  if $has{l_X11};
-  push(@x11_libs, 'Xext') if $has{l_Xext};
-
-  my @opengl_libs; # for non MS Windows
-  push(@opengl_libs, 'GL')  if $has{l_GL};
-  push(@opengl_libs, 'GLU') if $has{l_GLU};
-
-  #possible targets: im im_process im_jp2 im_fftw im_capture im_avi im_wmv im_fftw3 im_ecw
-  my @imtargets = qw[im im_process im_jp2 im_fftw];
-  if ($^O eq 'openbsd') {
-    warn "###WARN### Skipping im_process on OpenBSD"; # xxx TODO xxx
-    @imtargets = grep { $_ !~ /^(im_process)$/ } @imtargets;
-  }
-
-  #possible targets: cd_freetype cd_ftgl cd cd_pdflib cdpdf cdgl cdcontextplus cdcairo
-  my @cdtargets = qw[cd_freetype cd_ftgl cd cd_pdflib cdpdf cdgl];
-  @cdtargets = grep { $_ !~ /^(cd_ftgl|cdgl)$/ } @cdtargets unless $has{l_GLU};
-
-  #possible targets: iup iupcd iupcontrols iupim iupimglib iup_pplot iupgl
-  my @iuptargets = qw[iup iupcd iupcontrols iupim iupimglib iup_pplot iupgl];
-  @iuptargets = grep { $_ !~ /^(iupgl)$/ } @iuptargets unless ($has{windows} && $has{gl}) || ($has{glx});
-
-  #store debug info into ConfigData
-  $self->config_data('debug_has', \%has);
-  $self->config_data('debug_imtargets', \@imtargets);
-  $self->config_data('debug_cdtargets', \@cdtargets);
-  $self->config_data('debug_iuptargets', \@iuptargets);
-
-  my @makeopts  = qw[NO_DYNAMIC=Yes USE_NODEPEND=Yes];
-  #my @makeopts  = qw[NO_STATIC=Yes USE_NODEPEND=Yes];
-
-  #choose GUI subsystem, priorities if multiple subsystems detected: 1. Win32(cygwin), 2. GTK, 3. X11/Motif
-  my @libs;
-  my @build_opts;
-  my $build_target = '';
-
-  push(@build_opts, 'MS Windows native') if $has{windows}; #cygwin only
-  push(@build_opts, 'GTK2') if $has{gtk};
-  push(@build_opts, 'X11/Motif') if ($has{Xlib} && $has{Xm});
-
-  if (scalar(@build_opts) == 1) {
-    $build_target = $build_opts[0];
-  }
-  elsif (scalar(@build_opts) > 1) {
-    my $n = 1;
-    my $msg = "\nYou have the following build options available:\n" .
-              join("\n", map ($n++ . ") $_", @build_opts)) .
-	      "\nWhat do you wanna build?";
-    my $i = $self->prompt($msg, 1);
-    $build_target = $build_opts[$i-1];
-    die "###ERROR### Wrong selection!" unless $build_target;
-  }
-  else {
-    warn "###WARN### No supported GUI subsystem (Win32, GTK, X11/Motif) detected! (trying X11)";
-    $success = 0;
-    $build_target = 'X11/Motif';
+  
+  unless ($build_target) {
+    warn "###FATAL### No supported GUI subsystem (GTK2, X11/Motif) detected! (gonna exit)\n";
+    warn "### for GTK2 you need: gtk+-2.0, gdk-2.0, cairo + X11/Xlib.h\n";
+    warn "### for X11/Motif you need: -lXm, -lX11 + Xm/Xm.h, X11/Xlib.h\n";
+    die;
   }
+  
+  $self->config_data('info_gui_driver', $build_target);
 
   print STDERR "Build target=", ($build_target || ''), "\n";
-  if ($build_target eq 'MS Windows native') { #cygwin only
-    push(@makeopts, 'USE_WIN=Yes');
-    push(@makeopts, 'X11_LIBS='); #no X11 libs on cygwin
-    push(@libs, qw[gdi32 comdlg32 comctl32 winspool uuid ole32 oleaut32 opengl32 glu32]); #xxx ???glut
-    ($extra_cflags, $extra_lflags) = ('', '');
-  }
-  elsif ($build_target eq 'GTK2') {
+  if ($build_target eq 'GTK2') {
     push(@makeopts, 'USE_GTK=Yes');
+    push(@makeopts, 'USE_GDK=Yes');
+    push(@makeopts, 'USE_PKGCONFIG=Yes');
+    
+    #xxx maybe remove in the future (temporary fix)
+    my $gtk_base = $self->run_stdout2str(qw[pkg-config --variable=prefix gtk+-2.0]);
+    push(@makeopts, "GTK_BASE=$gtk_base") if $gtk_base;
+    push(@makeopts, "GTK=$gtk_base") if $gtk_base;
+
+    #handle existing freetype
+    if ($has{freetype}) { #xxx TODO: later replace with $has{freetype2}
+      @cdtargets = grep { $_ !~ /^(cd_freetype)$/ } @cdtargets;
+    }
+
     #detected libs
-    push(@makeopts, "GTK_BASE=$dir_gtk") if $dir_gtk;
     push(@makeopts, "X11_LIBS=" . join(' ', @x11_libs));
     push(@makeopts, "X11_LIB=$dir_x11_lib") if $dir_x11_lib;
     push(@makeopts, "X11_INC=$dir_x11_inc") if $dir_x11_inc;
@@ -223,6 +270,7 @@ sub build_binaries {
   }
   elsif ($build_target eq 'X11/Motif') {
     push(@makeopts, 'USE_X11=Yes');
+    push(@makeopts, 'USE_MOTIF=Yes');
     #additional X11 related libs
     push(@x11_libs, 'Xp')   if $has{l_Xp};
     push(@x11_libs, 'Xt')   if $has{l_Xt};
@@ -245,11 +293,8 @@ sub build_binaries {
   }
 
   #do the job
-  print STDERR "Gonna make these targets: " . join(' ', @iuptargets, @cdtargets, @imtargets) . "\n";
-  unless ($self->build_via_tecmake($build_out, $srcdir, \@makeopts, \@iuptargets, \@cdtargets, \@imtargets)) {
-    warn "###MAKE FAILED###";
-    $success = 0;
-  }
+  $success = $self->build_via_tecmake($build_out, $srcdir, \@makeopts, \@iuptargets, \@cdtargets, \@imtargets);
+  warn "###MAKE FAILED###" unless $success;
 
   #make a list of libs necessary to link with IUP and related libraries
   my %seen;
@@ -261,6 +306,7 @@ sub build_binaries {
     $success = 0;
   }
   foreach (@gl_l) {
+    print STDERR "Created lib: $_\n" if $self->notes('build_debug_info');
     if ($_ =~ /lib([a-zA-Z0-9\_\-\.]*?)\.(so|dylib|bundle|a|dll\.a)$/) {
       $seen{$1} = 1;
     }
@@ -270,34 +316,26 @@ sub build_binaries {
     }
   }
 
-  push(@libs, 'stdc++'); # -lstdc++ needed by Linux (at least)
-
-  print STDERR "Output libs: $_\n" foreach (sort keys %seen);
-  @libs = ( $self->sort_libs(keys %seen), @libs );
-
-  $self->config_data('linker_libs', \@libs);
+  push(@libs, 'stdc++') if $has{'l_stdc++'}; # -lstdc++ needed by Linux (at least)
+  my @iuplibs = $self->sort_libs(keys %seen);
+  $self->config_data('iup_libs', {map {$_=>1} @iuplibs} );
+  $self->config_data('linker_libs', [@iuplibs, @libs] );
   $self->config_data('extra_cflags', $extra_cflags);
   $self->config_data('extra_lflags', $extra_lflags);
 
-  die "###BUILD ABORTED###" unless $success;
-  print STDERR "Build finished sucessfully!\n";
-
-  #DEBUG: fail intentionally here if you want to see build details from cpan testers
-  #die "Intentionally failing";
+  print STDERR "Build finished!\n";
+  return $success;
 };
 
 sub build_via_tecmake {
   my ($self, $build_out, $srcdir, $mopts, $iuptgs, $cdtgs, $imtgs) = @_;
   $srcdir ||= 'src';
   my $prefixdir = rel2abs($build_out);
+
   my $make = $self->notes('gnu_make') || $self->get_make;
   die "###ERROR## make command not defined" unless $make;
-  my $im_si;
-  my $success = 1;
 
-  # save it for future use in ConfigData
-  $self->config_data('build_prefix', $prefixdir);
-  $self->config_data('debug_makeopts', $mopts);
+  my $success = 1;
 
   #create output directory structure
   mkdir "$prefixdir" unless -d "$prefixdir";
@@ -309,17 +347,9 @@ sub build_via_tecmake {
   if(-d "$srcdir/im/src") {
     print STDERR "Gonna build 'im'\n";
     chdir "$srcdir/im/src";
-    # some debug info
-    $im_si = $self->run_output_std($make, qw/-f tecmake.mak sysinfo MAKENAME= USE_NODEPEND=Yes/, @{$mopts}) if $self->notes('build_debug_info');    
     foreach my $t (@{$imtgs}) {
-      if ($self->notes('build_msgs')) {
-        $done{$t} = $self->run_output_std($make, $t, @{$mopts});
-      }
-      else {
-        $done{$t} = $self->run_output_on_error(undef, $make, $t, @{$mopts});
-      }
-      warn "###WARN### error [$?] during make $t" unless $done{$t};
-      $success = 0 unless $done{$t};
+      $done{"im:$t"} = $self->run_custom($make, $t, @{$mopts});
+      $success = 0 unless $done{"im:$t"};
     }
     copy($_, "$prefixdir/include/") foreach (glob("../include/*.h"));
     copy($_, "$prefixdir/lib/") foreach (glob("../lib/*/*"));
@@ -330,14 +360,8 @@ sub build_via_tecmake {
     print STDERR "Gonna build 'cd'\n";
     chdir "$srcdir/cd/src";
     foreach my $t (@{$cdtgs}) {
-      if ($self->notes('build_msgs')) {
-        $done{$t} = $self->run_output_std($make, $t, @{$mopts});
-      }
-      else {
-        $done{$t} = $self->run_output_on_error(undef, $make, $t, @{$mopts});
-      }
-      warn "###WARN### error [$?] during make $t" unless $done{$t};
-      $success = 0 unless $done{$t};
+      $done{"cd:$t"} = $self->run_custom($make, $t, @{$mopts});
+      $success = 0 unless $done{"cd:$t"};
     }
     copy($_, "$prefixdir/include/") foreach (glob("../include/*.h"));
     copy($_, "$prefixdir/lib/") foreach (glob("../lib/*/*"));
@@ -348,28 +372,18 @@ sub build_via_tecmake {
     print STDERR "Gonna build 'iup'\n";
     chdir "$srcdir/iup";
     foreach my $t (@{$iuptgs}) {
-      if ($self->notes('build_msgs')) {
-        $done{$t} = $self->run_output_std($make, $t, @{$mopts});
-      }
-      else {
-        $done{$t} = $self->run_output_on_error(undef, $make, $t, @{$mopts});
-      }
-      warn "###WARN### error [$?] during make $t" unless $done{$t};
-      $success = 0 unless $done{$t};
+      $done{"iup:$t"} = $self->run_custom($make, $t, @{$mopts});
+      $success = 0 unless $done{"iup:$t"};
     }
     copy($_, "$prefixdir/include/") foreach (glob("./include/*.h"));
     copy($_, "$prefixdir/lib/") foreach (glob("./lib/*/*"));
     chdir $self->base_dir();
   }
 
-  unless ($done{iup} && $done{iupim} && $done{iupcd}) {
-    warn "###WARN### essential libs not built!";
-    $success = 0;
-  }
-
-  print STDERR "Done: $done{$_} - $_\n" foreach (sort keys %done);
-  $self->config_data('debug_done', \%done);
-  $self->config_data('debug_si', $im_si);
+  # save it for future use in ConfigData
+  $self->config_data('build_prefix', $prefixdir);
+  $self->config_data('info_makeopts', $mopts);
+  $self->config_data('info_done', \%done);
 
   return $success;
 }
@@ -380,28 +394,23 @@ sub get_make {
   my $devnull = File::Spec->devnull();
   my @try = ($Config{gmake}, 'gmake', 'make', $Config{make});
   my %tested;
-  print STDERR "Gonna detect GNU make:\n";
-
-  if ($^O eq 'cygwin') {
-    print STDERR "- on cygwin always 'make'\n";
-    return 'make'
-  }
+  print STDERR "Gonna detect GNU make\n" if $self->notes('build_debug_info');
 
   foreach my $name ( @try ) {
     next unless $name;
     next if $tested{$name};
     $tested{$name} = 1;
-    print STDERR "- testing: '$name'\n";
+    print STDERR "- testing: '$name'\n" if $self->notes('build_debug_info');
     my $ver = `$name --version 2> $devnull`;
     if ($ver =~ /GNU Make/i) {
-      print STDERR "- found: '$name'\n";
+      print STDERR "- found: '$name'\n" if $self->notes('build_debug_info');
       return $name
     }
   }
 
   warn "###WARN### it seems we do not have GNU make, build is likely gonna fail!";
   return;
-  
+
   #print STDERR "- fallback to: 'make'\n";
   #return 'make';
 }
